#include "usb.h"

void usb_close(struct usb_device *dev)
{
 int c;

 for(c=0;c<dev->interfaces;c++)
 {
  if(dev->claimed[c])
   libusb_release_interface(dev->device, c);

  if(dev->kernel[c])
   libusb_attach_kernel_driver(dev->device, c);
 }

 if(dev->device)
  libusb_close(dev->device);
 
 libusb_exit(NULL);
}

int usb_open(struct usb_device *dev, uint8_t interfaces)
{
 int r=0, c;

 dev->device=NULL;
 memset(dev->kernel, 0, USB_MAX_INTERFACES);
 memset(dev->claimed, 0, USB_MAX_INTERFACES);

 if(interfaces==0||interfaces>USB_MAX_INTERFACES)
 {
  printf("Device interfaces should be between 1 and %d, reverting to default (1)\n", USB_MAX_INTERFACES);
  dev->interfaces=1;
 }
 else
  dev->interfaces=interfaces;

 if((r=libusb_init(NULL))<0)
 {
  printf("Failed to initialize libusb, error: %d\n", r);
  return 1;
 }

 if((dev->device=libusb_open_device_with_vid_pid(NULL, dev->vendor, dev->product))<=0)
 {
  printf("Could not find/open device %.4x:%.4x, error: %d\n", dev->vendor, dev->product, dev->device);
  dev->device=NULL; //not opened
  return 1;
 }

 for(c=0;c<dev->interfaces;c++)
 {
  if(libusb_kernel_driver_active(dev->device, c)==1)
  {
   if((r=libusb_detach_kernel_driver(dev->device, c))<0)
   {
    printf("Could not detach kernel driver from interface: %d, error: %d\n", c, r);
    usb_close(dev);
    return 1;
   }

   dev->kernel[c]=1;
  }

  if((r=libusb_claim_interface(dev->device, c))<0)
  {
   printf("Could not claim interface: %d, error %d\n", c, r);
   usb_close(dev);
   return 1;
  }

  dev->claimed[c]=1;
 }

 return 0;
}


void usb_transfer_reset(struct usb_transfer *transfer, uint16_t length, const unsigned char *data)
{
 uint16_t c;

 if(transfer->data)
  free(transfer->data); //cleanup data

 transfer->endpoint=LIBUSB_ENDPOINT_OUT;
 transfer->recipient=LIBUSB_RECIPIENT_DEVICE;
 transfer->type=LIBUSB_REQUEST_TYPE_STANDARD;

 transfer->request=LIBUSB_REQUEST_GET_STATUS;
 transfer->value=0x00;
 
 transfer->data=NULL;
 transfer->length=length;

 if(transfer->length) //data length provided
 {
  transfer->data=(unsigned char *)malloc(sizeof(unsigned char)*transfer->length); //allocate memory

  if(data) //data provided
  {
   for(c=0;c<transfer->length;c++) //copy data
    transfer->data[c]=data[c];
  }
  else
   memset(transfer->data, 0, transfer->length);
 }
}

int usb_control(struct usb_device *dev, struct usb_transfer *transfer)
{
 int r;

#ifdef USB_DEBUG
 printf("Control: endpoint: 0x%.4x | recipient: 0x%.4x | type: 0x%.4x = reqType: 0x%.4x, request: 0x%.4x, value: 0x%.4x, index: 0x%.4x\n", transfer->endpoint, transfer->recipient, transfer->type, transfer->endpoint|transfer->recipient|transfer->type, transfer->request, transfer->value, transfer->index);
 if(transfer->length)
 {
  printf("Send data(%d): 0x", transfer->length);
  for(r=0;r<transfer->length;r++)
  {
   printf("%.2x", transfer->data[r]);
   if((r+1)%4==0) printf(" 0x");
  }
  printf("\n");
 }
#endif

 if((r=libusb_control_transfer(dev->device, transfer->endpoint|transfer->recipient|transfer->type, transfer->request, transfer->value, transfer->index, transfer->data, transfer->length, USB_CONTROL_TIMEOUT))<0)
 {
  printf("Unable to control device, error %d\n", r);
  usb_transfer_reset(transfer, 0, NULL);
  return -r;
 }

 transfer->length=r; //store bytes received

#ifdef USB_DEBUG
 if(transfer->length)
 {
  printf("Receiced data(%d): 0x", transfer->length);
  for(r=0;r<transfer->length;r++)
  {
   printf("%.2x", transfer->data[r]);
   if((r+1)%4==0) printf(" 0x");
  }
  printf("\n");
 }
#endif

 return 0;
}

int usb_bulk(struct usb_device *dev, struct usb_transfer *transfer)
{
 int r, transferred;

#ifdef USB_DEBUG
 printf("Bulk: endpoint: 0x%.4x | recipient: 0x%.4x = endpoint: 0x%.4x, length: %d\n", transfer->endpoint, transfer->recipient, transfer->endpoint|transfer->recipient, transfer->length);
#endif


 if((r=libusb_bulk_transfer(dev->device, transfer->endpoint|transfer->recipient, transfer->data, transfer->length, &transferred, USB_BULK_TIMEOUT))<0)
 {
  if(r!=LIBUSB_ERROR_TIMEOUT)
  {
   printf("Unable to receive bulk data from device, error %d\n", r);
   usb_transfer_reset(transfer, 0, NULL);
   return -r;
  }
 }

 transfer->length=transferred; //store bytes received

#ifdef USB_DEBUG
 if(transfer->length)
 {
  printf("Receiced data(%d): 0x", transfer->length);
  for(r=0;r<transfer->length;r++)
  {
   printf("%.2x", transfer->data[r]);
   if((r+1)%4==0) printf(" 0x");
  }
  printf("\n");
 }
#endif

 return 0;
}

